package Sim;

import org.gillius.Util.CheckUtils;

/**
 * FixedLocationManager implements a LocationManager that always returns the same location. This can
 * be used to simulate nodes that never move.
 *
 * @author Jason Winnebeck
 */
public class FixedLocationManager implements LocationManager {
	private final Location location;

	/**
	 * Constructs a new FixedLocationManager that always returns the given Location from its
	 * getLocation method.
	 *
	 * @param location non-null Location
	 */
	public FixedLocationManager( Location location ) {
		CheckUtils.checkNull( location, "location" );
		
		this.location = location;
	}

	/**
	 * Returns the location produced by this manager at the given time t. The value of t passed in must
	 * be equal to or greater than the value of all previous t values passed into this method.
	 *
	 * @return non-null generated Location
	 */
	public Location getLocation( double t ) {
		return location;
	}
}
