package Sim;

import org.gillius.Util.CheckUtils;

/**
 * HardwareFrame represents a physical layer data transfer from one neighbor to the next.
 *
 * @author Jason Winnebeck
 */
public class HardwareFrame {
	private final int source;
	private final int dest;

	//TODO: should userData really be inside protocolData like it would be in a real network?
	private final ProtocolFrame protocolData;
	private final Packet userData;

	/**
	 * Broadcast address.
	 */
	public static final int BROADCAST = -1;

	/**
	 * Constructs a new HardwareFrame that will be broadcast from the node with "sourceId" and is meant
	 * for neighbor with "destId". The HardwareFrame contains the given data Packet.
	 *
	 * @param source       sending node
	 * @param dest         destination neighbor, or {@link #BROADCAST} if broadcast
	 * @param protocolData non-null routing protocol section of frame
	 * @param userData     nullable user packet data, if any attached
	 */
	public HardwareFrame( int source, int dest, ProtocolFrame protocolData, Packet userData ) {
		CheckUtils.checkNull( protocolData, "protocolData" );

		this.source = source;
		this.dest = dest;
		this.protocolData = protocolData;
		this.userData = userData;
	}

	public boolean isBroadcast() {
		return dest < 0;
	}

	public int getSize() {
		return protocolData.getSize() + ( ( userData != null ) ? userData.getSize() : 0 );
	}

	/**
	 * Returns {@link #getSize()} * 8.
	 */
	public int getSizeInBits() {
		return getSize() * 8;
	}

	public int getSource() {
		return source;
	}

	public int getDest() {
		return dest;
	}

	public ProtocolFrame getProtocolData() {
		return protocolData;
	}

	public Packet getUserData() {
		return userData;
	}

	public String toString() {
		return "HardwareFrame from " + source + " to " + dest + ", protocol header " + protocolData +
	         ", user data " + userData;
	}
}
