package Sim;
import java.util.ArrayList;
/**
 * A line object to construct a line based on two points.  This
 * object offers the slope, yintercept, available points on the
 * line and the intersection point given another line object.
 * 
 * @author Benjamin Willis
 *
 */
public class Line {
	private double slope;
	private double yintercept;
	private ArrayList<Location> knownPoints = new ArrayList<Location>();
	
	/**
	 * Construct a line based on two location objects.
	 * 
	 * @param a point 1 of this line
	 * @param b points 2 of the line
	 */
	public Line(Location a, Location b){
		knownPoints.add(a);
		knownPoints.add(b);
		//slope
		if(b.getX() - a.getX() == 0){
			slope = 0;
		}else{
			this.slope = (b.getY() - a.getY())/(b.getX() - a.getX());
		}
		//calcuate y intercept
		this.yintercept = a.getY() - slope * a.getX(); 
	}
	
	/**
	 * Get the intersection of this line and another.
	 * @param other
	 * @return
	 */
	public Location getIntersection(Line other){
		// m1x + b1 = m2x + b2
		//	or
		// x = (m2x + b2 - b1)/m1
		double x = (other.getSlope() + other.getYintercept() - this.yintercept) / this.slope;
		//plug in either eq for y
		double y = this.slope * x + this.yintercept;
		Location intersection = new Location(x,y);
		knownPoints.add(intersection);
		return intersection;
	}

	public ArrayList<Location> getKnownPoints() {
		return knownPoints;
	}

	public Boolean isOnLine(Location l){
		double closeToZero = l.getY() - slope*l.getX() + yintercept;
		//System.out.println("is on line?:"+closeToZero);
		return 1 > closeToZero && -1 < closeToZero;
	}
	
	public double getSlope() {
		return slope;
	}

	public double getYintercept() {
		return yintercept;
	}
}
