package Sim;

/**
 * PacketListener is an interface for objects wanting to listen for Packet events.
 *
 * @author Jason Winnebeck
 */
public interface PacketListener {
	/**
	 * Event called when a Packet is given to a Node for sending. This event only occurs once for any
	 * given Packet.
	 *
	 * @param p    non-null Packet
	 * @param time Time of origination.
	 */
	public void packetOriginated( Packet p, double time );

	/**
	 * Event called when a Packet is is passed to a new node, including the terminating node.
	 *
	 * @param p    non-null Packet
	 * @param time Time of origination.
	 */
	public void reportTotalHops(Packet p, int hops, double distance);
	
	/**
	 * Event called when a Packet arrives at its destination. This method can happen multiple times if
	 * there is packet duplication. The first time a Packet arrives, repeat is false. All subsequent
	 * events, if they occur, repeat is true.
	 *
	 * @param p      non-null Packet that arrived
	 * @param time   Time of arrival, which is after time of origination for this Packet
	 * @param repeat true if another copy of the Packet arrived, false if first time.
	 */
	public void packetArrived( Packet p, double time, boolean repeat );
}
