package Sim;

import java.util.Random;

/**
 * A location creator that is used to get random value locations
 * that are within the range specified in the constructor.
 * 
 * @author Benjamin Willis
 */
public class RandomLocationGenerator implements LocationGenerator{

	private Random rand;
	private Double max;
	
	/**
	 * Set the input random seed, max x and y for the returned location.
	 * 
	 * @param seed  Seed for the randomly generated locations
	 * @param xMax  Max x value that should be used in generating locations
	 * @param yMax  Max y value that should be used for generating locations
	 */
	public RandomLocationGenerator(Integer seed, Integer area) {
		this.rand = new Random(seed);
		this.max = (Double) Math.ceil(Math.sqrt(area));
	}
	/**
	 * Gets the next location based on the input paramters.
	 * 
	 * @return new random location based on x, y and seed inputs
	 */
	public Location getNext(){
		return new Location(rand.nextInt(max.intValue()),rand.nextInt(max.intValue()));
	}

}
