/*
 * ChampAlgorithm.java
 *
 * Created on May 5, 2007, 8:09 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package Sim.Routing.Champ;

import Sim.Node;
import Sim.Packet;
import Sim.Routing.AbstractRoutingAlgorithm;
import java.util.*;
import Sim.HardwareFrame;
import Sim.ProtocolFrame;


/**
 *
 * @author Travis
 */
public class ChampAlgorithm extends AbstractRoutingAlgorithm{
    private final int cacheSize = 5;  //authors claim 5 is ideal
    private final int numPaths = 2;    //authors say 2 performs well
    private static final double NO_ACK_TIMEOUT = .2;
    private static final double ROUTE_REQUEST_TIME = .5 ;  //arbitrary time to wait for route
    private static final int NETWORK_DIAMETER_EST = 8;  //may need to be adjusted  also in RREQTimeOutEvent
    private static final double RouteRepairTime = .2;
    private int currentSequenceNumber;
    private LinkedList <Packet> waitingForRoute = new LinkedList();
    private LinkedList <Packet> waitingToRequest= new LinkedList();
    private boolean needLog = false;
    private LinkedList <AckWaiter>ackWaiterList = new LinkedList();
    private int previousNode;
    private static int totalTerminated;
    public double lastRouteRequest = 0;  //will give us something to backoff from for these
    private static final double routeRequestBackoff = .1;
    private LinkedList <RouteRepairTimeout> routeRepairList = new LinkedList<RouteRepairTimeout>();
    
    
    private RouteCache routeCache = new RouteCache();
    private RouteRequestCache requestCache = new RouteRequestCache(numPaths);
    private LinkedList <HardwareFrame> dataCache = new LinkedList<HardwareFrame>(); //for storing packets that have been sent
    // private int currentSpot = 0;
    int nodeAddress;
    
    /*
     *handleRERR will handle a RERR message coming to this node
     *@param RERRHeader error
     *@param HardwareFrame in
     */
    private void handleRERR(RERRHeader error, HardwareFrame in){
        
        routeCache.removeSuccessor(error.getNodeSendingErr());  //remove that successor
        if(myNode.getAddress() == error.getSource()){                   //if we are source of packet
            RouteRequested routeReq = requestCache.getRoute(error.getSource(),
                    error.getDestination(),error.getSequenceNumber());
            
            if(routeCache.getNextHop(error.getDestination()) == null){   //if not next hop
                //no route need to find rout
                Destination des = null;
                for(int i = 0; i < dataCache.size(); i++){
                    NormalPacketHeader packetHeader = (NormalPacketHeader)
                    dataCache.get(i).getProtocolData();
                    Packet pkt = dataCache.get(i).getUserData();
                    if(error.getSource() == packetHeader.getSourceAddress()){
                        if(error.getDestination() == packetHeader.getDestination()){
                            if(error.getSequenceNumber() == packetHeader.getSequenceNumber()){
                                this.waitingToRequest.addFirst(pkt);  //do route request if we have pkt
                            }
                        }
                    }
                    
                }
                
                
            }
        } else if(routeCache.getNextHop(error.getDestination()) != null){  //we have a route to destination
            int i = this.seeIfInDataCache(error);
            if(i > -1){  //if its in cache then try to forward the data
                this.routeData(dataCache.get(i).getUserData(),
                                    routeCache.getNextHop(error.getDestination()));  //send the data
            }         
                            

        }
        
        else{               //no destination and we are not source
           int i = this.seeIfInDataCache(error);
           if(i > -1){
               NormalPacketHeader pktHeader = (NormalPacketHeader)
                    dataCache.get(i).getProtocolData();
               this.sendRERR(error.getSource(),
                       error.getDestination(),
                       error.getSequenceNumber(),
                       pktHeader.getPreviousHopAddress());
               
           }
           else{    //don't have packet'
               NextHop previousHop = routeCache.getNextHop(error.getSource());  //get hop toward source
               if(previousHop != null){
                        this.sendRERR(error.getSource(),
                       error.getDestination(),
                       error.getSequenceNumber(),
                       previousHop.getAddressAsInt());
               }
           }
            
        }
 
    }
    /*
     *forwardDataPacket will forward a data packet to the next hop
     *@param toForward is the packet in a frame to forward
     */
    public HardwareFrame forwardDataPacket(NormalPacketHeader toForward,
            HardwareFrame in){
        Packet packet = in.getUserData();   //get packet
        
        NextHop next = routeCache.getNextHop(packet.getDest());  //get next hop
        if(next == null){
            next = routeCache.getNextHop(packet.getDest());
            
        }
        HardwareFrame frame = null;
        if(next != null){             //send the packet
            frame = new HardwareFrame( myNode.getAddress(),
                    next.getAddressAsInt(),toForward,packet );  //send to next neibor
            
            
            myNode.sendFrame(frame);             //send this frame to next
            
            dataCache.add(frame);    //save packet in dataCache
            
            
            if(dataCache.size() >= cacheSize){
                dataCache.removeFirst();   //remove the first one
            }
            
            
            
        } else{    //we don't have a path and we should not be the source
            
            this.sendRERR(packet.getSource(),
                    packet.getDest(),
                    toForward.getSequenceNumber(),
                    this.previousNode);
        }
        return frame;
        
    }
    
    
    
    /*sendRERR will send a route error frame to indicate an error has occurred
     *@param int source
     *@param int destination
     *@param int sequenceNumber
     *@param int previousHop
     */
    public void sendRERR(int source, int destination,
            int sequenceNumber,int previousHop){
        
        RERRHeader error = new RERRHeader(source,  //source of packet
                destination,                       //destination of packet
                sequenceNumber,                    //sequence number
                previousHop,                       //previous hop
                myNode.getAddress());                          //this node
        HardwareFrame frame = new HardwareFrame( myNode.getAddress(),
                this.previousNode,
                error,null );  //send error to previous
        
        myNode.sendFrame(frame);  // actually send error;
    }
    
    
    
    /**
     *terminated will basically sent an ack back it it receives a packet
     *@HardwareFrame in
     */
    public void terminated(HardwareFrame in){
        NormalPacketHeader done = (NormalPacketHeader)in.getProtocolData();
        
        this.sendAck(done,in);
        totalTerminated++;  //static will count all that get terminated
        
    }
    
    /*arrived will handle messages arrive at this node
     *This method will determine what kind of message it
     *is.  Then call the appropriate function to handle it
     *@param HardwareFrame in
     */
    public void arrived(HardwareFrame in){
        ProtocolFrame header = in.getProtocolData();
        this.previousNode = in.getSource();
        
        
        
        if(header instanceof NormalPacketHeader){
            NormalPacketHeader toForward = (NormalPacketHeader) header;
            
            HardwareFrame sent = forwardDataPacket(toForward,in);
            this.sendAck(toForward,in);
            if(sent != null){
                AckWaiter ackWait = new AckWaiter( //list of acks
                        toForward.getSequenceNumber(), //sequence number
                        sent.getDest(),               //nexthop dest
                        toForward,                  //data packe
                        sent.getUserData(),           //packet
                        this.routeCache,            //routeCache
                        this,                       //champ
                        myNode);
                this.scheduleEvent(this.getTime() + this.NO_ACK_TIMEOUT,
                        ackWait);                   //node
                this.ackWaiterList.add(ackWait);
            }
            
            if(needLog){
                this.debug(in,in.getSource(),
                        in.getUserData().getSource());
            }
        }
        
        
        else if(header instanceof RERRHeader){
            RERRHeader error = (RERRHeader) header;
            if(error.getSequenceNumber() == 44){
                myNode.getLocation();
            }
            
            handleRERR(error,in);  //handle the error message
            
            if(needLog){
                this.debug(in,in.getSource(),
                        in.getUserData().getSource());
            }
        } else if(header instanceof RREQHeader){
            RREQHeader request = (RREQHeader) header;
            
            
            this.handleRouteRequestMsg(request.getSource(),
                    request.getDestination(),
                    request.getSequenceNumber(),
                    request.getPreviousHop(),
                    request.getForwardCount(),
                    request.getLastKnownDistance(),
                    request.getPropagationRange());
            if(needLog){
                this.debug(in,in.getSource(),
                        request.getSource());
            }
            
            
            
        } else if(header instanceof AckHeader){
            
            AckHeader ack = (AckHeader)header;
            for(int i = 0;i < ackWaiterList.size(); i++){
                if(ackWaiterList.get(i).isAcknowledged(
                        ack.getNodeAcking(),ack.getSequenceNumber())){
                    ackWaiterList.remove();
                    i = 0;   //just to make sure we check them all
                }
            }
            //this.ackList.add((AckHeader)header);  //save acks so they can be checked in AckWaiter
        } else if(header instanceof RREPHeader){
            RREPHeader reply = (RREPHeader) header;
            
            RouteRequested route = requestCache.getRoute(
                    reply.getSource(),
                    reply.getDest(),
                    reply.getSeqNum());
            if(route != null){
                handleRouteReplyMsg(reply.getSource(),
                        reply.getDest(),
                        reply.getSeqNum(),
                        reply.getNodeReplyFrom(),
                        reply.getPreviousHops(),
                        reply.getHopCount(),
                        reply.getAge(),
                        reply.getRouteLength());
            }
            if(needLog){
                this.debug(in,reply.getNodeReplyFrom(),
                        reply.getSource());
            }
        }
    }
    
    
    /**
     * Event when a Node wants to send a new Packet. The RoutingAlgorithm should bundle the Packet in a
     * HardwareFrame and send it through the Node.
     *
     * @param dest   non-null Node as the destination (packet.getDest() == dest.getAddress())
     * @param packet user packet data to send, with a destination address not equal to the address for
     *               the sending Node.
     */
    @Override
    public void newPacket( Node dest, Packet packet ){
        
        NextHop nextHop = routeCache.getNextHop(dest.getAddress());
        this.currentSequenceNumber++;
        
        NormalPacketHeader header = new NormalPacketHeader(currentSequenceNumber,
                packet.getSource(),myNode.getAddress(),packet.getDest());
        HardwareFrame frame = null;
        
        
        if(nextHop == null){
            frame = new HardwareFrame(packet.getSource(),
                    0,header,packet);
            this.currentSequenceNumber++;
            requestCache.addRoute(dest.getAddress(),
                    20,
                    null,
                    false,
                    currentSequenceNumber,
                    myNode.getAddress());
            
            if(waitingForRoute.size() == 0){
                sendRouteRequest(myNode.getAddress(),
                        dest.getAddress(),
                        -1,
                        currentSequenceNumber,
                        0,
                        NETWORK_DIAMETER_EST,
                        frame,
                        true);
                dest.getAddress();
                
                waitingForRoute.add(packet);  //save packet until a route comes in
            } else{
                this.waitingToRequest.add(packet); //save packet and request later
            }
        } else{
            System.out.println(totalTerminated);
            System.out.println("Able to send new packet!\n");
            frame = new HardwareFrame(packet.getSource(),
                    nextHop.getAddress(),header,packet);
            
            myNode.sendFrame(frame);
        }
        
        
        
    }
    
    
    
    
    /**RouteReplyRecved will handle a RouteReply message being
     *received by this node in the network.
     *@param source
     *@param destination
     *@param sequenceNumber
     *@param previousHop
     *@param previousHops
     *@param hopCount
     *@param age
     *@param formerLength
     */
    private void handleRouteReplyMsg(int source, int destination,
            int sequenceNumber, int nextHop,
            LinkedList <Integer> previousHops,
            int hopCount,double age, int forwardLength){
        
        
        if(source == myNode.getAddress()){    //yes we have found a route
            if(this.waitingToRequest.size() > 0){
                Packet pkt = waitingToRequest.getFirst();
                NormalPacketHeader normal =
                        new NormalPacketHeader(currentSequenceNumber,
                        waitingToRequest.getFirst().getSource(),
                        myNode.getAddress(),
                        waitingToRequest.getFirst().getDest());
                
                HardwareFrame hard = new
                        HardwareFrame(pkt.getSource(),
                        0,normal,pkt);
                this.sendRouteRequest(pkt.getSource(),
                        pkt.getDest(),
                        this.NETWORK_DIAMETER_EST,
                        currentSequenceNumber,
                        0,
                        this.NETWORK_DIAMETER_EST,
                        hard,
                        true);
                waitingToRequest.add(this.waitingToRequest.remove());
            }
            
            if(routeCache.getNextHop(destination) == null){
                routeCache.addDestination(2,destination,hopCount,nextHop);
            } else{
                routeCache.addSuccessor(destination,nextHop,forwardLength);
                
            }
            requestCache.setRouteStatus(source,destination,sequenceNumber,
                    true);  //hopefully stop RREQ
            for(int i =0; i < this.waitingForRoute.size(); i++){
                NextHop next = routeCache.getNextHop(
                        waitingForRoute.get(i).getDest());
                if(next != null){
                    
                    this.currentSequenceNumber++;
                    NormalPacketHeader header = new NormalPacketHeader(
                            this.currentSequenceNumber,
                            source,
                            this.getAddress(),
                            destination);
                    HardwareFrame frame = new HardwareFrame(this.getAddress(),
                            next.getAddressAsInt(),
                            header,
                            this.waitingForRoute.get(i));
                    waitingForRoute.remove(i);
                    forwardDataPacket(header,frame);
                    
                }
                
            }
            if(this.waitingToRequest.size()!= 0){
                currentSequenceNumber++;
                Packet requestingFor = waitingToRequest.get(0);
                RREQHeader request = new
                        RREQHeader(requestingFor.getSource(),
                        requestingFor.getDest(),
                        currentSequenceNumber,
                        -1,
                        0,
                        20,
                        this.NETWORK_DIAMETER_EST);
                HardwareFrame frame = new HardwareFrame(myNode.getAddress(),
                        HardwareFrame.BROADCAST,
                        request,waitingToRequest.get(0));
                
                this.waitingForRoute.add(waitingToRequest.get(0));
                waitingToRequest.remove(0); //move from one list to the other
                
                for(int j = 0; j < waitingToRequest.size(); j++) {
                    NextHop next = routeCache.getNextHop(waitingToRequest.get(j).getDest());
                    if(next != null){  //found a route for this too
                        this.routeData(waitingToRequest.get(j),
                                next);
                    }
                }
                
                this.sendRouteRequest(requestingFor.getSource(),
                        requestingFor.getDest(),
                        20,
                        currentSequenceNumber,
                        0,
                        this.NETWORK_DIAMETER_EST,
                        frame,
                        true);
                System.out.println(myNode.getAddress() + " Got a route and Requesting route to "+
                        requestingFor.getDest());
                
            }
            
        }
        
        
        
        
        if(previousHops != null){
            for(int i = 0; i < previousHops.size(); i++){
                Integer prev = previousHops.get(i);
                
                RouteRequested route = requestCache.getRoute(source,destination,
                        sequenceNumber);
                LinkedList <Integer> previoushops = route.getPreviousHops();
                if(!route.isRequestStatus()){  //if true don't send route reply'
                    boolean weAreAPrevious = false;
                    for(int j = 0; j < previousHops.size();j++){
                        if(myNode.getAddress() == previousHops.get(j).intValue()){
                            weAreAPrevious = true;
                        }
                    }
                    if(weAreAPrevious){
                        
                        this.sendRREP(source,
                                destination,
                                sequenceNumber,
                                previoushops,
                                forwardLength,
                                age,
                                ++hopCount);
                        
                        route.setRequestStatus(true);
                        routeCache.addDestination(2,destination,
                                forwardLength,
                                nextHop);
                        
                    }
                }
                
                
            }
        }
    }
    
    
    
    
    /**
     *initiateRouteRequest()
     *will try to found route from source to destination
     *@param source
     *@param int destination
     *@param lastKnownDistance
     *@param sequenceNumber
     *@param forwardcount
     *@param propagation range
     *@dataFrame hardwareframe
     *@boolean needTimeout
     */
    private void sendRouteRequest(int source,
            int destination,
            int lastKnownDistance,
            int sequenceNumber,
            int forwardCount,
            int propagationRange,
            HardwareFrame dataFrame,
            boolean needTimeout){
        
        RREQHeader header = new RREQHeader(source,       //source of RREQ
                destination,                             //destination
                sequenceNumber,                          //sequence
                myNode.getAddress(),                     //set previous hop to this one
                forwardCount,                       					//forward count starts at 0
                propagationRange,
                NETWORK_DIAMETER_EST);  //propRane and network diatmeter
        
        HardwareFrame frame = new HardwareFrame(myNode.getAddress(),
                HardwareFrame.BROADCAST, header, null );
        
        myNode.sendFrame(frame);
        if(needTimeout){
            HardwareFrame error = new HardwareFrame(
                    myNode.getAddress(),
                    HardwareFrame.BROADCAST,
                    new RERRHeader(source,
                    destination,sequenceNumber,myNode.getAddress(),myNode.getAddress()),
                    null);
            RREQTimeOutEvent timeout = new
                    RREQTimeOutEvent(frame,
                    dataFrame,
                    error,
                    needTimeout,
                    requestCache.getRoute(source,destination,sequenceNumber),
                    routeCache,
                    myNode);
            this.scheduleEvent(this.getTime() +this.ROUTE_REQUEST_TIME,timeout);
        }
        
        currentSequenceNumber++;  //increment sequence number
        
        
    }
    
    
    
    private void sendRouteRequest(int source,
            int destination,
            int lastKnownDistance,
            int sequenceNumber,
            int forwardCount,
            int propagationRange
            ){
        
        RREQHeader header = new RREQHeader(source,       //source of RREQ
                destination,                             //destination
                sequenceNumber,                          //sequence
                myNode.getAddress(),                     //set previous hop to this one
                forwardCount,                       					//forward count starts at 0
                propagationRange,
                NETWORK_DIAMETER_EST);  //propRane and network diatmeter
        
        HardwareFrame frame = new HardwareFrame(myNode.getAddress(),
                HardwareFrame.BROADCAST, header, null );
        
        
        myNode.sendFrame(frame);
    }
    
    
    
        /*
         *handleRouteRequestMsg will handle routeRequestMsg's that the nodes recieves
         *@param source
         *@param destination
         *@param sequenceNumber
         *@param previousHop
         *@param lastKnownDistance
         *@param propagationRange
         */
    private void handleRouteRequestMsg(int source,
            int destination,
            int sequenceNumber,
            int previousHop,
            int forwardCount,
            int lastKnownDistance,
            int propagationRange){
        boolean newRequest = false;
        
        if(destination == myNode.getAddress()){  //if destination
            if(requestCache.getRoute(source,destination,sequenceNumber) == null){
                LinkedList <Integer> newList = new LinkedList<Integer>();
                newList.add(new Integer(previousHop));
                requestCache.addRoute(destination,forwardCount,newList,
                        false,sequenceNumber,source);
            }
            if(requestCache.getRouteMinForwardCount(source,
                    destination,
                    sequenceNumber) >= forwardCount){ //if new request took shorter or equal path
                LinkedList<Integer> previousHops = new LinkedList<Integer>();
                previousHops.add(previousHop);
                this.sendRREP(source,
                        destination,
                        sequenceNumber,
                        previousHops,
                        forwardCount,
                        0,  //should be youngest route
                        0);   //since were at destination 0 hops
            }
        }
        
        else if(requestCache.getRoute(source,destination,sequenceNumber) == null){//if never seen request
            LinkedList<Integer> previousHops = new LinkedList<Integer>();
            previousHops.add(new Integer(previousHop));
            requestCache.addRoute(destination,forwardCount,   //add it to request Cache
                    previousHops,
                    false,sequenceNumber,source);
            NextHop next = routeCache.getNextHop(destination);
            
            //if we have next
            //&& next.distance is <= lastknownDistance
            //&& previous hop is not route ccahe for successor
            if(next != null && next.getDistance() <= lastKnownDistance &&
                    !routeCache.isSuccessor(previousHop,destination)){
                //then broadcast RREP
                this.sendRREP(source,
                        destination,
                        sequenceNumber,
                        previousHops,
                        next.getDistance()+1,
                        next.getAge(),
                        next.getDistance() + forwardCount + 1);
                requestCache.setRouteStatus(source,destination,sequenceNumber,true);
                
            } else if(propagationRange > 0){
                this.sendRouteRequest(source,
                        destination,
                        lastKnownDistance,
                        sequenceNumber,
                        ++forwardCount,
                        --propagationRange
                        );
            }
            
            
        }
        
        
        else if(myNode.getAddress() != source){   //any other node but dest or source
            if(requestCache.getRouteMinForwardCount(source,destination,
                    sequenceNumber) == forwardCount && requestCache.getRoute(
                    source,destination,sequenceNumber).isRequestStatus() == false){  //if forward count on new message better and not replied
                requestCache.addPreviousHopToRoute(source,
                        destination,sequenceNumber,previousHop);  //will add if not there
            }
            if(requestCache.getRouteMinForwardCount(source,
                    destination,sequenceNumber) > forwardCount){   //if better route
                requestCache.updateForwardCount(source,destination,
                        sequenceNumber,forwardCount);
                requestCache.replacePreviousHops(source,destination,sequenceNumber,
                        previousHop);
            }
            
            
        }
    }
    
    
    
    
    
    
    
    
        /*sendRREP will send a route reply to where specified
         *@param int originalSource
         *@param int finalDestination
         *@param int sequenceNumber
         *@param int previousHop
         *@param int fowardCount
         *@param int nodeSendingTo
         */
    private void sendRREP(int originalSource,
            int finalDestination,
            int sequenceNumber,
            LinkedList <Integer> previous,
            int forwardCount,
            double age,
            int hopCount){
        
        RREPHeader header = new RREPHeader(originalSource,  //source request came from
                finalDestination,                       //destination this node
                sequenceNumber,                    //sequence number of request
                myNode.getAddress(),
                previous,                          //at dest just one hop set
                hopCount,                                 //hopCount is 0
                age,                                 //age is 0
                forwardCount);                     //length of route
        
        HardwareFrame frame = new HardwareFrame(myNode.getAddress(),  //this nodes address
                previous.getFirst().intValue(),                     //node to send this frame too
                header,null);
        
        myNode.sendFrame(frame);
        
        
    }
        /*
         *routeData will create a frame with a given packet and
         *send it out of the node
         *@param packet
         *@param nextHop
         */
    private void routeData( Packet packet ,NextHop nextHop ) {
        
        NormalPacketHeader header = new NormalPacketHeader(currentSequenceNumber,
                packet.getSource(),myNode.getAddress(),
                nextHop.getAddressAsInt());
        
        HardwareFrame frame = new HardwareFrame( myNode.getAddress(),
                nextHop.getAddressAsInt(),header,packet );
        //probably going to need to send event if no ack
        AckWaiter ackWait = new AckWaiter(currentSequenceNumber,
                nextHop.getAddressAsInt(),
                header,
                packet,
                routeCache,
                this,
                myNode);
        this.ackWaiterList.add(ackWait);
        this.scheduleEvent(this.getTime() + this.NO_ACK_TIMEOUT,ackWait);
        
        myNode.sendFrame( frame );  //send the frame to the simulation
    }
    
    
    
    /**
     *routeRepair()  will send out RREQ to see if it can get a new path for
     *the packet.
     *@param source,
     *@param destination
     *@param sequenceNumber
     *@param forwardCount
     *@param lastKnownDistance
     *@param propagationRange
     *@param packetTryingToSend
     */
    public void routeRepair(int source,
            int destination,
            int sequenceNumber,
            int forwardCount,
            int lastKnownDistance,
            Packet packetTryingToSend){
        
        RREQHeader routeReq = new RREQHeader(myNode.getAddress(),  //use our node as source
                destination,     //where packet wants to go
                sequenceNumber,  //sequenceNumber for this
                myNode.getAddress(),        //node that is looking for a route
                forwardCount,    //this is supposed to be set to something special
                lastKnownDistance,
                2);
        NormalPacketHeader norHeader = new NormalPacketHeader(sequenceNumber,
                packetTryingToSend.getSource(),
                this.getAddress(),
                packetTryingToSend.getDest());
        
        HardwareFrame send = new HardwareFrame( myNode.getAddress(),
                HardwareFrame.BROADCAST,
                routeReq,
                null );
        
            
            myNode.sendFrame(send);  //send RREQ
            RouteRepairTimeout routeTimeout = new RouteRepairTimeout(sequenceNumber, norHeader,
                    packetTryingToSend,this);
            this.scheduleEvent( this.getTime()+this.RouteRepairTime, routeTimeout);
            
            
            
        
    }
        
        
        
        /**
         *getAddress()
         *@return address of node
         */
        public int getAddress(){
            return myNode.getAddress();
        }
        
        
        
        /*debug will output some messages that will hopefully help in debug
         */
        private void debug(HardwareFrame in, int source,int originalSource){
            ProtocolFrame header = in.getProtocolData();
            if(header instanceof RREPHeader){
                RREPHeader reply = (RREPHeader)header;
                
            }
            if(header instanceof RREQHeader){
                RREQHeader request = (RREQHeader) header;
                
            }
        }
        
        
        /**sendAck will send a dataPacket back to the previous hop
         *@param toForward
         *@param in
         */
        private void sendAck(NormalPacketHeader toForward,
                HardwareFrame in){
            AckHeader acknowledge = new AckHeader(toForward.getSequenceNumber(),
                    myNode.getAddress());
            HardwareFrame ack = new HardwareFrame(myNode.getAddress(),//from this node
                    in.getSource(),  //send back to where in frame from
                    acknowledge,
                    null);
            
            myNode.sendFrame(ack);  //send ack back to previous hop
            
        }
        
        public RouteCache getRouteCache() {
            return routeCache;
        }
        
        public LinkedList<HardwareFrame> getDataCache() {
            return dataCache;
        }

        
        /**
         *seeIfInDataCache
         *if it is found then return true;
         *@param error RERRHeader
         *@return index of packet in dataCache
         */
        private int seeIfInDataCache(RERRHeader error) {
            int i = 0;
            int toReturn = -1;  
            for( i = 0; i < dataCache.size(); i++){
                if(dataCache.get(i).getSource() == error.getSource()){
                    if(dataCache.get(i).getDest() == error.getDestination()){
                        NormalPacketHeader header = (NormalPacketHeader)
                        dataCache.get(i).getProtocolData();
                        if(header.getSequenceNumber() == error.getSequenceNumber()){
                            //and if we have the packet in the data cache forward it
                            toReturn = i;
                            
                        }
                    }
                }
            }
            return toReturn;
        }
}
        
        
        
        
        
    

