/*
 * RouteCache.java
 *
 * Created on May 5, 2007, 9:08 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package Sim.Routing.Champ;
import java.util.LinkedList;
import Sim.Node;

/**
 *
 * @author Travis
 */
public class RouteCache {
    private LinkedList <Destination> destinations;
    
    
    /*getNextHop() will return an NODE that is the next hop for the
     *destination.  If no path exists it will return null
     *@param destination
     *@return Node
     */
    public NextHop  getNextHop(int destination){
        NextHop nextHop = null;
        if(getDestinations() != null){
            for(int i = 0;i < getDestinations().size(); i++){
                if(getDestinations().get(i).getDestination().intValue() == destination){
                    nextHop = getDestinations().get(i).getLeastUsedNextHop();
                    if(nextHop != null){
                        nextHop.setAge(nextHop.getAge()+1);   //just going to increment age not exactly what it should be thouhg
                        nextHop.use(nextHop.getUsedCount()+1); 
                        i = getDestinations().size();  // get out of here
                    }
                }
            }
        }
        return nextHop;
    }
    
    
    /**
     *isSuccessor will return true is the node passed to it is 
     *successor to destination
     *@int node
     *@int destination
     *@return true if its a successor
     */
    public boolean isSuccessor(int node,int destination){
        boolean toReturn = false;
        if(getDestinations() != null){
            for(int i = 0;i < getDestinations().size(); i++){
                if(getDestinations().get(i).getDestination() == destination){
                     LinkedList <NextHop> successors = getDestinations().get(i).getSuccessors();
                     for(int j = 0; j < successors.size(); j++){
                         if(successors.get(j).getAddress() == node){
                            toReturn = true;
                         }
                     }
             
                }
            }
        }
        return toReturn;
    }
    
    /** Creates a new instance of RouteCache */
    public RouteCache() {
    }
    
    /**
     *addDestination will add a destination to the routeCache
     *@param numPaths max allowed paths
     *@param dest final destination
     *@param distance total distance
     *@successor next hop toward destinaion
     */
    public void addDestination(int numPaths, int dest, int distance,
            int successor){
        LinkedList<Destination> destinations =  this.getDestinations();
        if(destinations == null){   //if destinations doesn't exist create the list'
            destinations = new LinkedList<Destination>();
        }
        Destination destToAdd = new Destination(numPaths,dest,distance);
        LinkedList <NextHop> success = new LinkedList<NextHop>();    //make a list for successor
        success.add(new NextHop(successor,distance));                //add next hop to sucessro
        destToAdd.setSuccessors(success);
        destinations.add(destToAdd);   //add a destination
        
        //destinations.get(0).setSuccessors(success);                  //try to set successors
        this.setDestinations(destinations);
        
    }
    
    
    /**
     *addSuccessor
     *@param destination
     *@param successor next hop
     *@param finalDistance
     */
    public void addSuccessor(int destination, int successor, int finalDistance){
        int oldest = 0;
        double oldestAge =0;
        for(int i = 0; i < destinations.size(); i++){
            if(destinations.get(i).getDestination().intValue() == destination){
                Destination dest = destinations.get(i);
                if(dest.getSuccessors().size() >= dest.getNumberPaths()){
                    LinkedList <NextHop> successors = dest.getSuccessors();
                    for(int j = 0;j < successors.size(); j++){     //find index of older route
                        if(successors.get(i).getAge() > oldestAge){
                            oldest = i;
                            oldestAge = successors.get(i).getAge();
                        }
                    }
                    successors.set(oldest,new NextHop(successor,finalDistance));
                }
            }
        }
    }
    
    /*
     *removeDestination will remove the specified node from cache
     *@address of node to remove
     *@return distance to dest
     */
    public int removeSuccessor(int address){
        int distance = -1;
        if(getDestinations()!= null){
            for(int i = 0; i < getDestinations().size(); i++){
            
                if(getDestinations().get(i).getSuccessors() != null){
                    for(int j = 0; j  < getDestinations().get(i).getSuccessors().size(); j++){
                        if(getDestinations().get(i).getSuccessors().get(j).getAddressAsInt() == address){
                            distance = getDestinations().get(i).getSuccessors().get(j).getDistance();
                            getDestinations().get(i).getSuccessors().remove(j);  //remove the matching successor
                        }
                    }
                }
                
            }
        }
        return distance;
            
        
    }

    public LinkedList<Destination> getDestinations() {
        return destinations;
    }

    public void setDestinations(LinkedList<Destination> destinations) {
        this.destinations = destinations;
    }
    
    /**
     *setDestNextHop will set the next hop for this destination in the the route
     *Cache
     *@param destination
     *@param hop address of next hop
     *@param distance to final destination
     */
    public void setDestNextHop(int destination,int hop,int distanceToDest){
        for(int i = 0; i < destinations.size(); i++){
            Destination dest = destinations.get(i);
            if(dest.getDestination().intValue() == destination){ //if destination is the one we have information for
                LinkedList <NextHop> hops = dest.getSuccessors();
                if(hops.size() > dest.getNumberPaths()){   //will have too many paths
                    int oldest = 0;
                    for(int j = 0; j < hops.size(); j++){
                        if(hops.get(oldest).getAge() < hops.get(i).getAge()){
                            oldest = j;  //set oldest to the older route;
                        }
                    }
                    //should have found oldest
                    hops.remove(oldest);
                    hops.add(new NextHop(hop,distanceToDest));
                }
                else{
                    hops.add(new NextHop(hop,distanceToDest));
                }
                    
                
                
                
            }
        }
    }
    
    
    
    
}
