package Sim.Routing.DSR;

import Sim.ProtocolFrame;

/**
 * RouteError tells nodes about a broken link from one point to another.
 *
 * @author Jason Winnebeck
 */
public class RouteError implements ProtocolFrame {
	private final int linkStart;
	private final int linkEnd;

	public RouteError( int linkStart, int linkEnd ) {
		this.linkStart = linkStart;
		this.linkEnd = linkEnd;
	}

	/**
	 * Returns the first (starting) node of the broken link.
	 */
	public int getLinkStart() {
		return linkStart;
	}

	/**
	 * Returns the second (ending) node of the broken link.
	 */
	public int getLinkEnd() {
		return linkEnd;
	}

	/**
	 * Returns the "serialized" size of this portion of the packet, in bytes.
	 */
	public int getSize() {
		return 4+4;
	}

	public String toString() {
		return "RouteError: " + linkStart + "-" + linkEnd;
	}
}
