package org.gillius.Util;

import java.util.Collection;

/**
 * CheckUtils provides utitilies for precondition checking.
 *
 * @author Jason Winnebeck
 */
public class CheckUtils {
    /**
     * Checks if the given parameter is null, and if so throws an IllegalArgumentException with an
     * appropriate message.
     */
    public static void checkNull( Object parameter, String paramName )
        throws IllegalArgumentException {
        if ( parameter == null ) {
            throw new IllegalArgumentException( "Parameter " + paramName +
                                                " must not be null." );
        }
    }

    /**
     * Checks if the given parameter is null or has a size less than the given size, and if so,
     * throws an IllegalArgumentException with an appropriate message.
     */
    public static void checkSize( Collection c, String paramName, int size ) {
        checkNull( c, paramName );
        if ( c.size() < size ) {
            throw new IllegalArgumentException(
                "Parameter " + paramName + " must contain at least " + size +
                " elements, but it only has " + c.size() + '.' );
        }
    }

    /**
     * Checks if the given parameter is null or has a size less than the given size, and if so,
     * throws an IllegalArgumentException with an appropriate message.
     */
    public static void checkSize( Object[] arr, String paramName, int size ) {
        checkNull( arr, paramName );
        if ( arr.length < size ) {
            throw new IllegalArgumentException(
                "Parameter " + paramName + " must contain at least " + size +
                " elements, but it only has " + arr.length + '.' );
        }
    }
}
