package Sim;

import Sim.Routing.FloodingRoutingAlgorithm;
import junit.framework.TestCase;

/**
 * LoopbackTest
 *
 * @author Jason Winnebeck
 */
@SuppressWarnings( { "FieldCanBeLocal" } )
public class FloodTest extends TestCase {

	private Simulation sim;
	private Packet[] packets;

	@Override
	protected void setUp() throws Exception {
		super.setUp();

		SimulationParameters params = new SimulationParameters( 0, 0, 100000.0, 0.0, 100.0, 200 );

		sim = new Simulation( params );

		makeNode( 0,  0, 0 );
		makeNode( 1, 10, 0 );
		makeNode( 2, 20, 0 );
		makeNode( 3, 30, 0 );
		makeNode( 4, 40, 0 );

		packets = new Packet[] {
				new Packet( 0, 4, 100 ),
				new Packet( 4, 0, 200 ),
				new Packet( 2, 3, 50 ),
				new Packet( 3, 4, 10 )
		};
	}

	private Node makeNode( int addr, double x, double y ) {
		LocationManager lm = new FixedLocationManager( new Location( x, y ) );
		FloodingRoutingAlgorithm alg = new FloodingRoutingAlgorithm();
		alg.setSimulation( sim );
		Node ret = new Node( addr, sim, lm, alg, 11.0, new NodeMonitor() );
		sim.getWorld().addNode( ret );
		return ret;
	}

	public void testFloodSimple() throws Exception {
		Packet p = new Packet( 0, 4, 100 );
		send( p );

		while ( sim.next() ) {
		}

		assertTrue( p.hasArrived() );
	}

	public void testFloodMultiple() throws Exception {
		for ( Packet packet : packets ) {
			send( packet );

			while ( sim.next() ) {
			}

			assertTrue( packet.hasArrived() );
		}
	}

	public void testFloodMultiRoute() throws Exception {
		makeNode( 98, 15, 0 );
		makeNode( 99, 50, 0 );

		for ( Packet packet : packets ) {
			send( packet );

			while ( sim.next() ) {
			}

			assertTrue( packet.hasArrived() );
		}
	}

	public void testFloodComplex() throws Exception {
		//If we implement collisions, this test will fail.

		for ( Packet packet : packets )
			send( packet );

		while ( sim.next() ) {
		}

		for ( Packet packet : packets )
			assertTrue( packet.hasArrived() );
	}

	private void send( Packet packet ) {
		sim.getWorld().getNode( packet.getSource() ).sendNewPacket( packet );
	}
}