package Sim.Routing.DSR;

import junit.framework.TestCase;

/**
 * TestRoute
 *
 * @author Jason Winnebeck
 */
public class TestRoute extends TestCase {
	private Route route;

	protected void setUp() throws Exception {
		super.setUp();
		route = new Route( 1, 2, 3, 4, 5 );
	}

	public void testAccessors() {
		assertEquals( 5, route.numHops() );
		assertTrue( route.containsAddress( 3 ) );
		assertEquals( 1, route.getFirstHop() );
		assertEquals( 5, route.getLastHop() );

		assertEquals( route, route );

		assertEquals( "Route [1, 2, 3, 4, 5]", route.toString() );

		assertTrue( route.containsLink( 1, 2 ) );
		assertTrue( route.containsLink( 2, 3 ) );
		assertTrue( route.containsLink( 3, 4 ) );
		assertTrue( route.containsLink( 4, 5 ) );

		assertFalse( route.containsLink( 1, 3 ) );
		assertFalse( route.containsLink( 2, 1 ) );
		assertFalse( route.containsLink( 9, 12 ) );

		assertEquals( route, route.getRouteFrom( 1 ) );
		assertEquals( new Route( 3, 4, 5 ), route.getRouteFrom( 3 ) );
		assertEquals( new Route( 5 ), route.getRouteFrom( 5 ) );
		assertNull( route.getRouteFrom( 6 ) );
	}

	public void testReverse() {
		Route rev = route.reverse();

		assertEquals( new Route( 5, 4, 3, 2, 1 ), rev );

		assertEquals( 5, rev.numHops() );
		assertTrue( rev.containsAddress( 3 ) );
		assertEquals( 5, rev.getFirstHop() );
	}

	public void testAddHop() {
		Route add = route.addHop( 6 );

		assertEquals( new Route( 1, 2, 3, 4, 5 ), route );
		assertEquals( new Route( 1, 2, 3, 4, 5, 6 ), add );

		assertEquals( 6, add.numHops() );
		assertEquals( 1, add.getFirstHop() );
		assertTrue( add.containsAddress( 6 ) );
	}

	public void testRemoveHop() {
		Route rem = route.removeFirstHop();

		assertEquals( new Route( 1, 2, 3, 4, 5 ), route );
		assertEquals( new Route( 2, 3, 4, 5 ), rem );

		assertEquals( 4, rem.numHops() );
		assertEquals( 2, rem.getFirstHop() );
		assertFalse( rem.containsAddress( 1 ) );

		assertEquals( new Route( 1, 2, 3, 4 ), route.removeLastHop() );
		assertEquals( new Route( 1, 2, 3 ), route.removeLastHop().removeLastHop() );
	}

	public void testMultiple() {
		Route result = route.removeFirstHop().removeFirstHop().addHop( 6 ).removeFirstHop().reverse();

		assertEquals( new Route( 1, 2, 3, 4, 5 ), route );
		assertEquals( new Route( 6, 5, 4 ), result );

		result = route.reverse().removeFirstHop().addHop( 7 );
		assertEquals( new Route( 1, 2, 3, 4, 5 ), route );
		assertEquals( new Route( 4, 3, 2, 1, 7 ), result );
	}
}
